#!/usr/bin/env python
import csv
import glob
import os
import shutil
import subprocess
import textwrap
from functools import partial
from multiprocessing import Pool
from tempfile import NamedTemporaryFile

from Bio import SeqIO

from phylofisher import help_formatter


def bash_command(cmd):
    """Function to run bash commands in a shell"""
    command_run = subprocess.call(cmd, shell=True, executable='/bin/bash')#,
                                #  stderr=subprocess.DEVNULL, stdout=subprocess.DEVNULL)
    if command_run == 0:
        return True
    else:
        return False


def mkdir_and_cd(dir_name):
    try:
        os.mkdir(dir_name)
    except FileExistsError:
        pass
    os.chdir(dir_name)


def delete_gaps_stars(root):
    """Removes -'s and *'s from alignments"""
    file = f'{args.input}/{root}.fas'
    file_name = f'{root}.aa'
    with open(file_name, 'w') as res:
        for record in SeqIO.parse(file, 'fasta'):
            res.write(f'>{record.name}\n{str(record.seq).replace("-", "").replace("*", "")}\n')


def x_to_dash(file):
    """Replaces X's in alignments with -'s"""
    file_name = f'{os.path.basename(file).split(".")[0]}.pre_bmge'
    with open(f'{file_name}', 'w') as res:
        for record in SeqIO.parse(file, 'fasta'):
            res.write(f'>{record.name}\n{str(record.seq).replace("X", "-")}\n')


def read_full_proteins(core):
    full_prots = {}
    for record in SeqIO.parse(f'{args.output}/prequal/{core}.aa.filtered', 'fasta'):
        full_prots[record.name] = record.seq
    return full_prots


def good_length(trimmed_aln, threshold):
    core = trimmed_aln.split('.')[0]
    full_proteins = read_full_proteins(core)
    original_name = f'{core}.length_filtered'
    length = None
    with open(original_name, 'w') as res:
        for record in SeqIO.parse(trimmed_aln, 'fasta'):
            if length is None:
                length = len(record.seq)
            coverage = len(str(record.seq).replace('-', '').replace('X', '')) / len(record.seq)
            if coverage > threshold:
                res.write(f'>{record.description}\n{full_proteins[record.name]}\n')
            else:
                print('deleted:', record.name, coverage)


def mk_checkpoint_tmp(files):
    if os.path.isfile(f'checkpoint.tmp') is False:
        with open(f'checkpoint.tmp', 'w') as outfile:
            header = 'gene,prequal,mafft1,divvier1,bmge,mafft2,divvier2,trimal,raxml\n'
            outfile.write(header)
            for file in files:
                outfile.write(f'{file}{9 * ",0"}\n')
    else:
        pass


def get_checkpoints():
    checks = {}
    if os.path.isfile(f'{args.output}/checkpoint.tmp') is True:
        with open(f'checkpoint.tmp', 'r') as infile:
            for line in infile:
                line = line.strip()
                checks[line.split(',')[0]] = line.split(',')[1:]

    return checks


def update_checkpoints(root, prog):
    filename = f'{args.output}/checkpoint.tmp'
    tempfile = NamedTemporaryFile(mode='w', delete=False)

    fields = ['gene', 'prequal', 'mafft1', 'divvier1', 'bmge', 'mafft2', 'divvier2', 'trimal', 'raxml']

    with open(filename, 'r') as csvfile, tempfile:
        reader = csv.DictReader(csvfile, fieldnames=fields)
        writer = csv.DictWriter(tempfile, fieldnames=fields)
        # Writes header to the output file
        writer.writerow(next(reader))
        for row in reader:
            if row['gene'] == str(root):
                row[prog] = 1
            row = {'gene': row['gene'], 'prequal': row['prequal'], 'mafft1': row['mafft1'],
                   'divvier1': row['divvier1'], 'bmge': row['bmge'], 'mafft2': row['mafft2'],
                   'divvier2': row['divvier2'], 'trimal': row['trimal'], 'raxml': row['raxml']
                   }
            writer.writerow(row)
    shutil.move(tempfile.name, filename)


def prepare_analyses(checks, root):
    if (args.threads / file_count) > 1:
        threads = int(args.threads / file_count)
    else:
        threads = 1
    checks = checks[root]

    for i, check in enumerate(checks):
        # prequal
        if i == 0 and check == '0':
            mkdir_and_cd(f'{args.output}/prequal')
            delete_gaps_stars(root)
            status = bash_command(f'prequal {root}.aa')
            if status:
                update_checkpoints(root, 'prequal')
            else:
                break

        # Length Filtration
        elif i == 1 and check == '0':
            mkdir_and_cd(f'{args.output}/length_filtration')
            # mafft1
            mkdir_and_cd('mafft')
            # MAFFT - length filtration
            status = bash_command(
                f'mafft --thread {threads} --globalpair --maxiterate 1000 --unalignlevel 0.6 '
                f'{args.output}/prequal/{root}.aa.filtered > {root}.aln')
            if status:
                update_checkpoints(root, 'mafft1')
            else:
                break

        elif i == 2 and check == '0':
            mkdir_and_cd(f'{args.output}/length_filtration')
            mkdir_and_cd(f'{args.output}/length_filtration/divvier')
            # Divvier - length filtration
            status = bash_command(f'divvier -mincol 4 -partial {args.output}/length_filtration/mafft/{root}.aln')
            if status:
                shutil.move(f'../mafft/{root}.aln.partial.fas', f'./{root}.aln.divvy.fas')
                shutil.move(f'../mafft/{root}.aln.PP', f'./{root}.aln.PP')
                update_checkpoints(root, 'divvier1')
            else:
                break

        elif i == 3 and check == '0':
            mkdir_and_cd(f'{args.output}/length_filtration')
            mkdir_and_cd(f'{args.output}/length_filtration/bmge')
            # outputs pre_bmge
            x_to_dash(f'{args.output}/length_filtration/divvier/{root}.aln.divvy.fas')
            # BMGE
            status = bash_command(f'bmge -t AA -g 0.3 -i {root}.pre_bmge -of {root}.bmge')
            if status:
                good_length(trimmed_aln=f'{root}.bmge', threshold=0.5)
                update_checkpoints(root, 'bmge')
            else:
                break

        # mafft2
        elif i == 4 and check == '0':
            mkdir_and_cd(f'{args.output}/mafft')
            status = bash_command(
                f'mafft --thread {threads} --globalpair --maxiterate 1000 --unalignlevel 0.6 '
                f'{args.output}/length_filtration/bmge/{root}.length_filtered > {root}.aln2')
            if status:
                update_checkpoints(root, 'mafft2')
            else:
                break

        # divvier2
        elif i == 5 and check == '0':
            mkdir_and_cd(f'{args.output}/divvier')
            status = bash_command(f'divvier -mincol 4 -partial {args.output}/mafft/{root}.aln2')
            if status:
                shutil.move(f'../mafft/{root}.aln2.partial.fas', f'./{root}.aln2.divvy.fas')
                shutil.move(f'../mafft/{root}.aln2.PP', f'./{root}.aln2.PP')
                update_checkpoints(root, 'divvier2')
            else:
                break

        # trimal
        elif i == 6 and check == '0':
            mkdir_and_cd(f'{args.output}/trimal')
            status = bash_command(f'trimal -in {args.output}/divvier/{root}.aln2.divvy.fas -gt 0.01 -out {root}.final')
            if status:
                update_checkpoints(root, 'trimal')
            else:
                break

        # raxml
        elif args.no_tree is False and i == 7 and check == '0':
            mkdir_and_cd(f'{args.output}/RAxML')

            files = glob.glob(f'*{root}*')
            for f in files:
                try:
                    os.remove(f)
                except FileNotFoundError:
                    pass

            status = bash_command(f'raxmlHPC-PTHREADS-AVX2 -T {threads} -m PROTGAMMALG4XF -f a '
                                  f'-s {args.output}/trimal/{root}.final -n {root}.tre -x 123 -N 100 -p 12345')
            if status:
                update_checkpoints(root, 'raxml')
                mkdir_and_cd(f'{args.output}/trees')
                shutil.copy(f'{args.output}/RAxML/RAxML_bipartitions.{root}.tre',
                            f'{args.output}/trees/RAxML_bipartitions.{root}.tre')
                shutil.copy(f'{args.output}/trimal/{root}.final',
                            f'{args.output}/trees/{root}.final')
                shutil.copy(f'{args.output}/length_filtration/bmge/{root}.bmge',
                            f'{args.output}/trees/{root}.trimmed')
            else:
                break

    return root, checks


if __name__ == '__main__':
    description = 'Aligns, trims, and builds single gene trees from unaligned gene files.'
    usage = 'sgt_constructor.py -i path/to/input/ [OPTIONS]'
    parser, optional, required = help_formatter.initialize_argparse(name='sgt_constructor.py',
                                                                    desc=description,
                                                                    usage=usage)

    # Optional Arguments
    optional.add_argument('-t', '--threads', metavar='N', type=int, default=1,
                          help=textwrap.dedent("""\
                          Desired number of threads to be utilized.
                          Default: 1
                          """))
    optional.add_argument('--no_tree', action='store_true',
                          help=textwrap.dedent("""\
                          Do NOT build single gene trees.
                          Length filtration and trimmming only.
                          """))

    args = help_formatter.get_args(parser, optional, required, pre_suf=False, inp_dir=True)

    # Parallelization of prepare_analyses function
    args.input = os.path.abspath(args.input)
    roots = [os.path.basename(file).split('.')[0] for file in os.listdir(args.input) if file.endswith('.fas')]
    args.output = os.path.abspath(args.output)
    mkdir_and_cd(args.output)
    mk_checkpoint_tmp(roots)
    checkpoints = get_checkpoints()
    file_count = len(roots)

    processes = args.threads
    if file_count < args.threads:
        processes = file_count

    prep_analyses = partial(prepare_analyses, checkpoints)
    with Pool(processes=processes) as p:
        all_checks = p.map(prep_analyses, roots)



#!/usr/bin/env python
import configparser
import os
import sys
import textwrap
from pathlib import Path

from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord

from phylofisher import help_formatter


def check_metadata():
    """
    Checks to make sure taxa provided in to_collapse.csv are actually in the meta data
    """
    with open(args.input, 'r') as infile:
        to_collapse = set()
        for taxon in [line.strip().split(',')[1:] for line in infile]:
            to_collapse.update(taxon)

    taxon_dict = dict.fromkeys(to_collapse, False)

    with open(metadata, 'r') as infile:
        for line in infile:
            line = line.strip()

            short_name = line.split('\t')[0]
            if short_name in to_collapse:
                taxon_dict[short_name] = True

    for taxon in taxon_dict.keys():
        if taxon_dict[taxon] is False:
            print(f'{taxon} is not in the dataset.')

    if False in taxon_dict.values():
        sys.exit('Taxa not collapsed. Please fix errors above.')


def collapse():
    ortho_dir = f'{dfo}/orthologs'
    gene_files = [os.path.join(ortho_dir, gene) for gene in os.listdir(ortho_dir) if gene.endswith('.fas')]
    with open(args.input, 'r') as infile:
        collapse_dict = dict()
        for line in infile:
            collapse_dict[line.strip().split(',')[0]] = line.strip().split(',')[1:]
    empty_record = SeqRecord(Seq(''),
                             id='')

    for gene_file in gene_files:
        record_dict = {k: empty_record for k in collapse_dict.keys()}
        records = []
        with open(gene_file, 'r') as infile:
            for record in SeqIO.parse(infile, 'fasta'):
                for key in collapse_dict.keys():
                    if record.id in collapse_dict[key]:
                        if len(record.seq) > len(record_dict[key].seq):
                            record.id = key
                            record.description = ''
                            record_dict[key] = record
                    else:
                        records.append(record)

        for record in record_dict.values():
            records.append(record)

        with open(gene_file, 'w') as outfile:
            SeqIO.write(records, outfile, 'fasta')


if __name__ == '__main__':
    description = 'Collapses dataset entries into a single taxon'
    parser, optional, required = help_formatter.initialize_argparse(name='taxon_collapser.py',
                                                                    desc=description,
                                                                    usage='taxon_collapser.py [OPTIONS] -i <taxa>')

    required.add_argument('-i', '--input', type=str, metavar='to_collapse.csv', default=None,
                          help=textwrap.dedent("""\
                          Path to CSV formatted file containing taxa to be collapsed.
                          Example Row: "CollapsedName,Taxon1,Taxon2,...,TaxonN"""))
    args = help_formatter.get_args(parser, optional, required,
                                   out_dir=False, pre_suf=False, inp_dir=False)

    # Congig Parser
    config = configparser.ConfigParser()
    config.read('config.ini')
    dfo = str(Path(config['PATHS']['database_folder']).resolve())
    metadata = str(os.path.join(dfo, 'metadata.tsv'))

    check_metadata()
    collapse()



#!/usr/bin/env python
import os
import textwrap
from collections import Counter

import matplotlib.pyplot as plt
import pandas as pd
from ete3 import Tree

from phylofisher import help_formatter

plt.rcParams["figure.figsize"] = (10, 5)


def bipartitions(tree):
    bipar = set()
    all_ = frozenset(tree.get_leaf_names())
    for node in tree.traverse('preorder'):
        if node.is_leaf() is False:
            taxa = frozenset(node.get_leaf_names())
            if len(taxa) < (len(all_) - len(taxa)):
                if len(taxa) > 1:
                    bipar.add(taxa)
            else:
                rest = taxa.symmetric_difference(all_)
                if len(rest) > 1:
                    bipar.add(rest)
    return bipar


def support(trees):
    bootstrap = []
    n_trees = 0
    for line in open(trees):
        line = line.strip('')
        tree = Tree(line)
        n_trees += 1
        bootstrap = bootstrap + list(bipartitions(tree))
    norm_bootstrap = {}
    counted = dict(Counter(bootstrap))
    for key, value in counted.items():
        norm_bootstrap[key] = value / n_trees
    return norm_bootstrap


def get_support(group, supp_dict):
    query = frozenset(group)
    if query in supp_dict:
        return supp_dict[query]
    else:
        return 0


def parse_groups(input_file):
    query_dict = {}
    for line in open(input_file):
        line = line.strip()
        group = line.split(':')[0]
        orgs = line.split(':')[1]
        query_dict[group] = [org.strip() for org in orgs.split(',')]
    return query_dict.items()


def file_to_series(file):
    print(file)
    sup_dict = support(file)
    group_sup = {}
    for group, orgs in queries:
        group_sup[group] = get_support(orgs, sup_dict)
    s = pd.Series(group_sup)
    print(s)
    return s


def parse_bss():
    columns = []
    n = 0
    for line in open(args.bs_files):
        line = line.strip()
        column = file_to_series(line.strip())
        column.name = os.path.basename(line)
        # column.name = f'Step {n}'
        columns.append(column)
        n += 1
    return columns


def main():
    columns = parse_bss()
    df = pd.DataFrame(columns)
    # plot_df.sort_index(inplace=True)
    df.plot()
    plt.legend(loc=0, prop={'size': 6})
    plt.xticks(range(len(df.index)), list(df.index), rotation=45)
    # plt.tight_layout()
    plt.savefig("test_0toN.pdf")
    df.to_csv("test_out.csv")
    return df


if __name__ == "__main__":
    description = 'Calculates the observed occurrences of clades of interest in bootstrap trees.'
    parser, optional, required = help_formatter.initialize_argparse(name='bipartition_examiner.py',
                                                                    desc=description,
                                                                    usage='bipartition_examiner.py '
                                                                          '[OPTIONS] -i /path/to/input/')

    # Required Arguments
    required.add_argument('-b', '--bs_files', required=True, type=str, metavar='',
                          help=textwrap.dedent("""\
                          Path bootstrap files.
                          """))
    required.add_argument('-g', '--groups', type=str, required=True, metavar='',
                          help=textwrap.dedent("""\
                          groups
                          """))

    # Optional Arguments
    optional.add_argument('-on', '--output_name', type=str, metavar='out', default='out',
                          help=textwrap.dedent("""\
                          """))

    args = help_formatter.get_args(parser, optional, required, pre_suf=False, inp_dir=False)

    queries = parse_groups(args.groups)
    main()



#!/usr/bin/env python
import configparser
import os
import shutil
import textwrap
from pathlib import Path

from Bio import SeqIO

from phylofisher import help_formatter


def parse_ortholog_tsv():
    """
    Parses select_orthologs.tsv (output of select_orthologs.py) to determine which genes are to be included in
    the final dataset.
    :return:
    """
    with open('select_orthologs.tsv', 'r') as infile:
        infile.readline()
        genes_to_include = []
        for line in infile:
            line = line.strip()
            if 'In-Group Completeness' in line:
                gene, _, _, _, include = line.split('\t')
            else:
                gene, _, include = line.split('\t')
            if include == 'yes':
                genes_to_include.append(gene)

    return genes_to_include


def subset_orthologs():
    """
    Subsets orthologs from the Database into a new directory to be included in the final dataset
    :return:
    """
    # Creates output dir if it doesn't already exist
    if os.path.isdir(args.output) is False:
        os.makedirs(args.output)

    # Genes to include in the final dataset
    genes = parse_ortholog_tsv()
    # List of paths to ortholog files
    files = [os.path.join(orthologs_dir, x) for x in os.listdir(orthologs_dir) if x.endswith('.fas')]
    # Copies gene files to the output dir
    for gene in genes:
        for file in files:
            if gene == os.path.basename(file).split('.')[0]:
                src = file
                dest = f'{args.output}/{os.path.basename(file)}'
                shutil.copy(src, dest)


def parse_taxa_tsv():
    """
    Parses select_taxa.tsv (output of select_taxa.py) to determine which taxa are to be included in
    the final dataset.
    :param tsv_file:
    :return:
    """
    with open('select_taxa.tsv', 'r') as infile:
        infile.readline()
        taxa_to_include = []
        for line in infile:
            line = line.strip()
            taxon, _, _, _, include = line.split('\t')
            if include == 'yes':
                taxa_to_include.append(taxon)

    return taxa_to_include


def subset_taxa():
    """
    Subsets taxa from the Database into a new directory to be included in the final dataset
    :return:
    """
    taxa = parse_taxa_tsv()

    files = [file for file in os.listdir(args.output)]
    for file in files:
        with open(f'{args.output}/{file}', 'r') as infile, open(f'{args.output}/tmp', 'w') as outfile:
            records = []
            for record in SeqIO.parse(infile, 'fasta'):
                if args.chimeras:
                    pass
                elif record.description in taxa:
                    records.append(record)

            SeqIO.write(records, outfile, 'fasta')

        shutil.move(f'{args.output}/tmp', f'{args.output}/{file}')


if __name__ == '__main__':
    description = 'Subsets taxa and orthologs to be included in super matrix construction'
    parser, optional, required = help_formatter.initialize_argparse(name='prep_final_dataset.py',
                                                                    desc=description,
                                                                    usage='prep_final_dataset.py '
                                                                          '[OPTIONS]')

    # Optional Arguments
    optional.add_argument('--chimeras', type=str, metavar='orthologs.tsv', default=None,
                          help=textwrap.dedent("""\
                          Path to chimeras.tsv. This will collapses taxa listed in chimera.tsv into a chimera 
                          keeping the longest sequence for each gene.
                          """))

    args = help_formatter.get_args(parser, optional, required, inp_dir=False, pre_suf=False)

    config = configparser.ConfigParser()
    config.read('config.ini')
    dfo = str(Path(config['PATHS']['database_folder']).resolve())
    orthologs_dir = f'{dfo}/orthologs'

    if os.path.isdir(args.output) is False:
        os.mkdir(args.output)

    if os.path.isfile('select_orthologs.tsv'):
        subset_orthologs()
    else:
        files = [os.path.basename(file) for file in os.listdir(orthologs_dir)]
        for file in files:
            shutil.copy(f'{orthologs_dir}/{file}', f'{args.output}/{file}')

    if os.path.isfile('select_taxa.tsv'):
        subset_taxa()



#!/usr/bin/env python
import configparser
import csv
import os
import textwrap
from glob import glob
from pathlib import Path

from Bio import SeqIO

from phylofisher import help_formatter


def fasta_cleaner(file, org_set):
    records = list(SeqIO.parse(file, 'fasta'))
    with open(file, 'w') as res:
        for record in records:
            if record.name.split('.')[0] not in org_set:
                res.write(f'>{record.name}\n{record.seq}\n')


def delete_homologs(org_set):
    for folder in ['orthologs', 'paralogs']:
        files = glob(os.path.join(dfo, folder) + '/*.fas')
        for file in files:
            fasta_cleaner(file, org_set)


def parse_metadata():
    meta = os.path.join(dfo, 'metadata.tsv')
    with open(meta, 'r') as f:
        reader = csv.reader(f, delimiter='\t')
        lines = list(reader)
        return lines


def parse_input(infile):
    file_contents = set()
    with open(infile, 'r') as infile:
        for line in infile:
            line = line.strip()
            file_contents.add(line)

    return list(file_contents)


def delete_group_org(orgs_=None, groups=None):
    meta = os.path.join(dfo, 'metadata.tsv')
    if groups:
        group_set = set(groups)
    else:
        group_set = set()

    if orgs_:
        orgs = set(orgs_)
    else:
        orgs = set()

    lines = parse_metadata()
    orgs_to_del = set()
    with open(meta, 'w') as out_file:
        res = csv.writer(out_file, delimiter='\t')
        for line in lines:
            if line[2] in group_set:
                orgs_to_del.add(line[0])
            elif line[3] in group_set:
                orgs_to_del.add(line[0])
            elif line[0] in orgs:
                orgs_to_del.add(line[0])
            else:
                res.writerow(line)
    delete_homologs(orgs_to_del)


# TODO input as a file
if __name__ == '__main__':
    description = 'Deletes taxa and/or taxonomic groups from the database'
    parser, optional, required = help_formatter.initialize_argparse(name='purge.py',
                                                                    desc=description,
                                                                    usage='purge.py [OPTIONS] -i <in_dir>')

    # Optional Arguments
    optional.add_argument('-o', '--orgs', type=str, metavar='groups.txt',
                          help=textwrap.dedent("""\
                              Path to text file containing Unique IDs of organisms for deletion.
                              Example:
                              UniqueID-1
                              UniqueID-2
                              """))
    optional.add_argument('-g', '--tax_groups', metavar='groups.txt', type=str,
                          help=textwrap.dedent("""\
                              Path to text file containing taxonomic groups for deletion.
                              Example:
                              Group1
                              Group2
                              """))

    in_help = 'Path to database directory'
    args = help_formatter.get_args(parser, optional, required, pre_suf=False, in_help=in_help, out_dir=False)

    config = configparser.ConfigParser()
    config.read('config.ini')
    dfo = str(Path(config['PATHS']['database_folder']).resolve())
    multi_input = os.path.abspath(config['PATHS']['input_file'])

    if args.orgs:
        orgs = parse_input(args.orgs)
        delete_group_org(orgs_=orgs)
    if args.tax_groups:
        tax_groups = parse_input(args.tax_groups)
        delete_group_org(groups=tax_groups)





#!/usr/bin/env python
import os
import textwrap

import matplotlib.pyplot as plt
import pandas as pd
import scipy.cluster.hierarchy as shc
from Bio import SeqIO
from Bio.SeqUtils.ProtParam import ProteinAnalysis
from ete3 import Tree

from phylofisher import help_formatter


def distance_matrix2tree(Z, names):
    """Return tree representation for distance matrix"""
    n = Z.shape[0] + 1
    i2n = [0] * (2 * n - 1)
    t = Tree()
    for i, (idx1, idx2, dist, sample_count) in enumerate(Z):
        idx1, idx2 = int(idx1), int(idx2)
        # create Tree object for tips / leaves
        if idx1 < n:
            i2n[idx1] = Tree(name=names[idx1])
        if idx2 < n:
            i2n[idx2] = Tree(name=names[idx2])
        # create new node
        t = Tree()
        # normalise distance
        dist1 = dist - i2n[idx1].get_farthest_leaf()[1]
        dist2 = dist - i2n[idx2].get_farthest_leaf()[1]
        # add children
        t.add_child(i2n[idx1], dist=dist1)
        t.add_child(i2n[idx2], dist=dist2)
        # store
        i2n[n + i] = t
    return t


def make_plot():
    df = pd.read_csv(f'{args.output}/aa_comp.tsv', sep="\t")
    df = df.set_index('Taxon')

    # Todo: Make plot prettier
    plt.figure(figsize=(25, 10))
    plt.title("AA Composition Hierarchical Clustering")
    plt.xticks(rotation=90)
    z = shc.linkage(df, method='ward')
    t = distance_matrix2tree(z, df.index.values)
    t.write(outfile=f'{args.output}/distance_matrix.tre')
    shc.dendrogram(z, labels=df.index.values, color_threshold=0)
    plt.tight_layout()
    plt.savefig(f'{args.output}/AA_Composition_Hierarchical_Clustering.pdf')


if __name__ == '__main__':
    description = 'Calculates amino acid composition of the supplied super matrix'
    parser, optional, required = help_formatter.initialize_argparse(name='aa_comp_calculator.py',
                                                                    desc=description,
                                                                    usage='aa_comp_calculator.py '
                                                                          '[OPTIONS] -i <matrix>')

    required.add_argument('-i', '--input', required=True, type=str, metavar='matrix',
                          help=textwrap.dedent("""\
                          Path to input matrix for analysis.
                          """))
    # required.add_argument('-f', '--input_format', required=True, type=str, metavar='matrix',
    #                       help=textwrap.dedent("""\
    #                       Path to input matrix for analysis.
    #                       """))

    args = help_formatter.get_args(parser, optional, required, pre_suf=False, inp_dir=False)

    peptides = ['A', 'G', 'P', 'S', 'T', 'C', 'F', 'W', 'Y', 'H', 'R', 'K', 'M', 'I', 'L', 'V', 'N', 'D', 'E', 'Q']

    if not os.path.isdir(args.output):
        os.mkdir(args.output)

    with open(args.input, 'r') as infile, open(f'{args.output}/aa_comp.tsv', 'w') as outfile:
        outfile.write('Taxon\t' + '\t'.join(peptides) + '\n')

        # Reads in input file
        for record in SeqIO.parse(infile, format='phylip-relaxed'):
            outfile.write(f'{record.id}\t')
            analysed_seq = ProteinAnalysis(str(record.seq))
            count_dict = analysed_seq.count_amino_acids()
            length = len(str(record.seq).replace("-", "").replace("X", "").replace("*", ""))
            out_str = ''

            # Loops through peptides and checks to see if it is in count_dict
            for pep in peptides:
                if pep in count_dict.keys():
                    out_str += f'{float(count_dict[pep]) / length}\t'
                else:
                    out_str += '0\t'

            outfile.write(out_str.strip() + '\n')

    make_plot()



#!/usr/bin/env python
import os
from collections import Counter

import matplotlib.patches as mpatches
import matplotlib.pyplot as plt
import matplotlib.ticker as mtick
import numpy as np
import pandas as pd


def parse_aligns(args, input_dir):
    """
    Parses all fasta files in the user-given input directory
    Input: NONE
    Output: dictionary with genes a keys and SeqIO iterator as values
    """
    files = [os.path.join(input_dir, x) for x in os.listdir(input_dir) if x.endswith('.fas')]
    all_taxa_records = {}
    for file in files:
        gene = os.path.basename(file).split('.')[0]
        with open(file, 'r') as infile:
            for line in infile:
                line = line.strip()
                if line.startswith('>'):
                    taxon = line[1:]
                    if len(taxon.split('_')) == 4:
                        taxon = taxon.split('_')[0]

                    if taxon not in all_taxa_records.keys():
                        all_taxa_records[taxon] = [gene]
                    else:
                        all_taxa_records[taxon].append(gene)

    to_df = []
    for taxon in all_taxa_records:
        for gene in all_taxa_records[taxon]:
            to_df.append([taxon, gene])

    my_df = pd.DataFrame(to_df, columns=['UniqueID', 'Orthologs'])
    bin_mat = pd.crosstab(my_df.UniqueID, my_df.Orthologs)

    return bin_mat


def parse_metadata(metadata, input_meta=False):
    """
    Parses metadata.csv to get all org names in each group and subtax
    Input: NONE
    Output: (1) dictionary with groups/subtax as keys and sets of orgs in those groups/subtax as values
            (2) list of all orgs in metadata
    """
    taxa_dict = {}
    with open(metadata, 'r') as infile:
        for line in infile:
            line = line.strip()
            if "Unique ID" not in line:
                if input_meta:
                    _, _, org, group, subtax, _, long_name, _, _ = line.split('\t')
                else:
                    org, long_name, group, subtax, _, _ = line.split('\t')
                taxa_dict[org] = [group, subtax, long_name]

    return taxa_dict


def completeness(args, input_dir, genes=True):
    """
    Computes completeness of genes based on either all taxa_comp_df or a set of taxa_comp_df provided by user
    """

    matrix = parse_aligns(args, input_dir)
    taxa_count, gene_count = matrix.shape
    gene_comp = matrix.sum().divide(other=taxa_count)
    taxa_comp = matrix.sum(axis=1).divide(other=gene_count)

    if genes:
        return matrix
    else:
        return taxa_comp, len(gene_comp)


def make_plot(s, plot_name, y_count, genes=True):
    """
    Creates a plot of gene completeness. Plot is a bar chart with genes sorted from highest to lowest completeness on
        the x-axis and percent complete on the y-axis. Completeness is calculated as percent of taxa_comp_df the gene is present
        in.
    Input: Pandas DataFrame, plot name
    Output: PDF of plot
    """

    if genes:
        calculated_s = 'Ortholog'
        calculated_p = 'Orthologs'
        calc_by = 'Taxa'
    else:
        calculated_s = 'Taxon'
        calculated_p = 'Taxa'
        calc_by = 'Orthologs'

    s = s.sort_values(ascending=False)
    # Dictionary with completeness as key and color as threshold. Color will be used to color bars that are above
    #   threshold.
    colors_threshold = {0.0: 'coral', 0.1: 'black', 0.2: 'lime', 0.3: 'maroon', 0.4: 'khaki',
                        0.5: 'purple', 0.6: 'orange', 0.7: 'olive', 0.8: 'grey', 0.9: 'navy'}
    colors_threshold_inv = {v: k for k, v in colors_threshold.items()}

    # Assigns color to genes depending on the completeness. Based off dict above
    s_colors = [''] * len(s.index)
    for i, row in enumerate(s):
        for j, threshhold in enumerate(colors_threshold.keys()):
            if row >= threshhold:  # Assigns color based on completeness value
                s_colors[i] = colors_threshold[threshhold]
    df = s.to_frame()
    df['colors'] = s_colors  # Adds color list as a column in this DataFrame
    df_colors = tuple([x for x in df['colors']])  # Converts list to tuple because df.plot requires tuple not list
    threshold_counts = Counter(df['colors'])

    # initailizng plot
    fig = plt.figure()
    ax = fig.add_subplot(111)
    df.plot(kind='bar',
            width=0.85,
            color=[df_colors],
            figsize=(35, 10),
            legend=False,
            fontsize=8,
            ax=ax)
    ax.set_title(f'{calculated_s} Completeness by {calc_by}', fontsize=30)

    # Legend
    legend_data = []
    for i, x in enumerate(sorted(colors_threshold.keys(), reverse=True)):
        if i == len(threshold_counts):
            break
        if i == 0:
            label_str = f'Completeness ≥ {round(x * 100)}% ({threshold_counts[colors_threshold[x]]} {calculated_p})'
        else:
            label_str = (f'{round(x * 100)}% ≤ Completeness < {round((x + 0.1) * 100)}%'
                         f' ({threshold_counts[colors_threshold[x]]} {calculated_p})')
        legend_data.append(mpatches.Patch(color=colors_threshold[x],
                                          label=label_str))

    ax.legend(handles=legend_data, facecolor='white', framealpha=1)

    # Y-axis settings and formatting
    ax.set_ylabel(f'Percent Complete ({y_count} {calc_by})', labelpad=15, fontsize=20)
    ax.yaxis.set_major_formatter(mtick.PercentFormatter(1.0))
    ax.set_ylim(0, 1)
    ax.yaxis.set_ticks(np.arange(0, 1.1, 0.1))
    ax.tick_params(axis='y', which='both', labelleft=True, labelright=True)

    # X-axis settingsand formattings
    ax.set_xlabel(f'{calculated_p} ({len(df.index)})', labelpad=15, fontsize=20)

    # Horizontal Lines
    for x in threshold_counts.keys():  # Iterate through colors with counts being the values
        plt.axhline(y=colors_threshold_inv[x], xmin=0, xmax=1, color=x, zorder=0)
    plt.axhline(y=1, xmin=0, xmax=1, color='black', zorder=0)

    # Top X-Axis
    gene_num = len(df.index)
    ax2 = ax.twiny()
    ax2.set_xlim(ax.get_xlim())
    ax2.set_xticks(range(-1, gene_num, 5))
    ax2.set_xticklabels(range(0, gene_num, 5))
    ax2.xaxis.set_minor_locator(mtick.MultipleLocator(1))

    # Graphic rendering
    fig.set_tight_layout(True)
    fig.savefig(f'{plot_name}.pdf')
    fig.show()
